=== modified file 'Makefile'
--- Makefile	2012-01-11 08:06:03 +0000
+++ Makefile	2012-01-10 11:16:56 +0000
@@ -1,4 +1,2 @@
 check:
 	python -m unittest grackle.tests.test_client
-
-.PHONY: check

=== added file 'grackle-create-instance'
--- grackle-create-instance	1970-01-01 00:00:00 +0000
+++ grackle-create-instance	2012-01-20 08:26:17 +0000
@@ -0,0 +1,17 @@
+#!/usr/bin/env python
+
+from optparse import OptionParser
+
+from grackle.server.model import create_schema
+
+
+parser = OptionParser(usage="usage: %prog [options] <hostname> <keyspace>")
+parser.add_option(
+    "--clobber", action="store_true", default=False, dest="clobber",
+    help="clear the keyspace before initialising it")
+options, args = parser.parse_args()
+
+if len(args) != 2:
+    parser.error("hostname and keyspace required")
+
+create_schema(args[0], args[1], clobber=options.clobber)

=== modified file 'grackle/client.py'
--- grackle/client.py	2012-01-11 14:04:41 +0000
+++ grackle/client.py	2012-01-20 08:26:17 +0000
@@ -1,50 +1,13 @@
 import httplib
-import simplejson
-from urlparse import urlunparse
-from urllib import (
-    quote,
-    urlencode,
-)
-
-
-
-class GrackleClient:
-
-    def __init__(self, host, port):
-        self.host = host
-        self.port = port
-        self.netloc = '%s:%d' % (host, port)
-
-    def archive_url(self, archive_name, query):
-        path = '/%s' % quote(archive_name)
-        query_string = urlencode(query)
-        return urlunparse(('http', self.netloc, path, '', query_string, ''))
-
-    def _get_connection(self):
-        return httplib.HTTPConnection(self.host, self.port)
-
-    def _verb_archive(self, verb, archive_name, query, body=None):
-        url = self.archive_url(archive_name, query)
-        connection = self._get_connection()
-        connection.request(verb, url, body)
-        return connection.getresponse()
-
-    def put_message(self, archive_name, key, file_obj):
-        response = self._verb_archive(
-            'POST', archive_name, {'key': key}, file_obj.read())
-        data = response.read()
-        if response.status == httplib.BAD_REQUEST:
-            raise Exception('wtf')
-        elif response.status == httplib.CREATED:
-            return
-        else:
-            raise Exception('!!')
-
-    def get_messages(self, archive_name, message_ids=None):
-        parameters = {}
-        if message_ids is not None:
-            parameters['message_ids'] = message_ids
-        query = {'parameters': simplejson.dumps(parameters)}
-        response = self._verb_archive('GET', archive_name, query)
-        data = response.read()
-        return simplejson.loads(data)
+
+def put_message(archive_name, file_obj):
+    connection = httplib.HTTPConnection('localhost', 8435)
+    connection.request('POST', '/archive/%s' % archive_name, file_obj.read())
+    response = connection.getresponse()
+    data = response.read()
+    if response.status == httplib.BAD_REQUEST:
+        raise Exception('wtf')
+    elif response.status == httplib.CREATED:
+        return
+    else:
+        raise Exception('!!')

=== added directory 'grackle/server'
=== added file 'grackle/server/__init__.py'
=== added file 'grackle/server/cassandra.py'
--- grackle/server/cassandra.py	1970-01-01 00:00:00 +0000
+++ grackle/server/cassandra.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,41 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+"""Things to ease working with cassandra."""
+
+from pycassa.pool import InvalidRequestError
+
+
+def workaround_1779(callable, *args, **kwargs):
+    """Workaround cassandra not being able to do concurrent schema edits.
+
+    The callable is tried until it does not raised InvalidRequestException
+    with why = "Previous version mismatch. cannot apply."
+
+    :param callable: The callable to call.
+    :param args: The args for it.
+    :param kwargs: The kwargs for it.
+    :return: The result of calling the callable.
+    """
+    while True:
+        # Workaround https://issues.apache.org/jira/browse/CASSANDRA-1779:
+        # Cassandra cannot do concurrent schema changes.
+        try:
+            return callable(*args, **kwargs)
+            break
+        except InvalidRequestError as e:
+            if e.why != 'Previous version mismatch. cannot apply.':
+                raise

=== added file 'grackle/server/model.py'
--- grackle/server/model.py	1970-01-01 00:00:00 +0000
+++ grackle/server/model.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,80 @@
+# Copyright (c) 2012 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+import datetime
+import uuid
+
+import pycassa
+from pycassa.system_manager import (
+    LEXICAL_UUID_TYPE,
+    SystemManager,
+    TIME_UUID_TYPE,
+    )
+
+from grackle.server.cassandra import workaround_1779
+
+
+def create_schema(host, keyspace, clobber=False):
+    mgr = SystemManager(host)
+
+    if clobber:
+        for cf in mgr.get_keyspace_column_families(keyspace):
+            mgr.drop_column_family(keyspace, cf)
+
+    try:
+        workaround_1779(
+            mgr.create_column_family, keyspace, 'message',
+            key_validation_class=LEXICAL_UUID_TYPE)
+        workaround_1779(
+            mgr.create_column_family, keyspace, 'archive_message',
+            comparator_type=TIME_UUID_TYPE,
+            default_validation_class=LEXICAL_UUID_TYPE)
+        pass
+    finally:
+        mgr.close()
+
+
+class CassandraConnection(object):
+
+    def __init__(self, keyspace, host):
+        self._keyspace = keyspace
+        self._host = host
+        self._connection = self._connect()
+        self.messages = self._column_family('message')
+        self.archive_messages = self._column_family('archive_message')
+
+    def _connect(self):
+        return pycassa.connect(self._keyspace, self._host)
+
+    def _column_family(self, name):
+        return pycassa.ColumnFamily(self._connection, name)
+
+    def add_message(self, archive_uuid, message):
+        message_uuid = uuid.uuid4()
+        self.messages.insert(
+            message_uuid,
+            {'date_created': datetime.datetime.utcnow().isoformat() + 'Z',
+             'content': message,
+             })
+        self.archive_messages.insert(
+            archive_uuid, {datetime.datetime.now(): message_uuid})
+        return message_uuid
+
+    def get_messages(self, archive_uuid):
+        ids = self.archive_messages.get(
+            archive_uuid, column_count=10, column_reversed=True).values()
+        messages = self.messages.multiget(ids)
+        return [messages[id]['content'] for id in ids]

=== added directory 'grackle/server/testing'
=== added file 'grackle/server/testing/__init__.py'
=== added file 'grackle/server/testing/cassandra.py'
--- grackle/server/testing/cassandra.py	1970-01-01 00:00:00 +0000
+++ grackle/server/testing/cassandra.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,54 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+"""Test helpers for working with cassandra."""
+
+import os.path
+
+from fixtures import Fixture, TempDir
+from pycassa.system_manager import SystemManager
+
+from grackle.server.cassandra import workaround_1779
+from grackle.server.model import create_schema
+
+
+class TemporaryKeyspace(Fixture):
+    """Create a temporary keyspace.
+
+    The keyspace is named after a tempdir.
+    """
+
+    def setUp(self):
+        super(TemporaryKeyspace, self).setUp()
+        tempdir = self.useFixture(TempDir())
+        self.keyspace = os.path.basename(tempdir.path)
+        self.mgr = SystemManager()
+        workaround_1779(self.mgr.create_keyspace, self.keyspace,
+            replication_factor=1)
+        self.addCleanup(workaround_1779, self.mgr.drop_keyspace,
+            self.keyspace)
+
+
+class TemporaryDB(Fixture):
+    """Create a temporary usable OOPS DB.
+
+    The keyspace for it is at self.keyspace.
+    """
+
+    def setUp(self):
+        super(TemporaryDB, self).setUp()
+        self.keyspace = self.useFixture(TemporaryKeyspace()).keyspace
+        create_schema('localhost', self.keyspace)

=== added directory 'grackle/server/tests'
=== added file 'grackle/server/tests/__init__.py'
=== added file 'grackle/server/tests/test_wsgi.py'
--- grackle/server/tests/test_wsgi.py	1970-01-01 00:00:00 +0000
+++ grackle/server/tests/test_wsgi.py	2012-01-12 10:52:19 +0000
@@ -0,0 +1,41 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+from unittest import TestCase
+
+from webtest import TestApp
+
+from grackle.server.wsgi import app
+
+
+class TestRoot(TestCase):
+
+    def test_get(self):
+        test_app = TestApp(app.wsgifunc())
+        r = test_app.get('/')
+        self.assertEqual(200, r.status_int)
+        r.mustcontain('Service endpoint for Grackle.')
+
+
+class TestArchive(TestCase):
+
+    def test_get(self):
+        test_app = TestApp(app.wsgifunc())
+        r = test_app.get('/archive/foo')
+        self.assertEqual(200, r.status_int)
+        r.mustcontain('Service for archive foo.')

=== added file 'grackle/server/wsgi.py'
--- grackle/server/wsgi.py	1970-01-01 00:00:00 +0000
+++ grackle/server/wsgi.py	2012-01-20 08:26:17 +0000
@@ -0,0 +1,68 @@
+# Copyright (c) 2011 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public
+# License along with this program. If not, see
+# <http://www.gnu.org/licenses/>.
+
+__metaclass__ = type
+
+import web
+
+from grackle.server.model import CassandraConnection
+
+conn = None
+
+
+class root:
+
+    def GET(self):
+        web.header('Content-Type', 'text/plain')
+        return (
+            'Service endpoint for Grackle.\n\n'
+            'When you want mail that was archived in Grackle, you can get it '
+            'from Grackle')
+
+
+class archive:
+
+    def GET(self, name):
+        web.header('Content-Type', 'text/plain')
+        args = web.input()
+        if not getattr(args, 'op', None):
+            return self.index(name)
+
+        if args.op == 'get_messages':
+            return self.get_messages(name)
+
+    def POST(self, name):
+        data = web.data()
+        conn.add_message(name, data)
+        web.ctx.status = '201 Created'
+
+    def index(self, name):
+        return ('Service for archive %s.' % name)
+
+    def get_messages(self, name):
+        return repr(conn.get_messages(name))
+
+
+urls = (
+    '/', 'root',
+    '/archive/(.+)', 'archive',
+    )
+
+app = web.application(urls, globals(), autoreload=False)
+
+if __name__ == "__main__":
+    conn = CassandraConnection('grackle', 'localhost/9160')
+    app.run()

=== modified file 'grackle/tests/test_client.py'
--- grackle/tests/test_client.py	2012-01-11 14:04:41 +0000
+++ grackle/tests/test_client.py	2012-01-10 14:03:53 +0000
@@ -5,43 +5,27 @@
 import httplib
 import os
 from signal import SIGKILL
-import simplejson
 from StringIO import StringIO
 from unittest import TestCase
-from urlparse import urlparse
-from urlparse import parse_qs
 
 from testtools import ExpectedException
 
-from grackle.client import (
-    GrackleClient,
-    )
-
-
-class ForkedFake:
-
-    def __init__(self, port, messages=None):
+from grackle import client
+
+
+class Forked:
+
+    def __init__(self, func_or_method):
+        self.func_or_method = func_or_method
         self.pid = None
-        self.port = port
-        self.messages = messages
-        self.read_end, self.write_end = os.pipe()
-
-    def is_ready(self):
-        os.write(self.write_end, 'asdf')
 
     def __enter__(self):
         pid = os.fork()
-        if pid == 0:
-            self.start_server()
-        self.pid = pid
-        os.read(self.read_end, 1)
-        return
+        if pid != 0:
+            self.pid = pid
+            return
+        self.func_or_method()
 
-    def start_server(self):
-        service = HTTPServer(('', self.port), FakeGrackleRequestHandler)
-        service.messages = self.messages
-        self.is_ready()
-        service.serve_forever()
 
     def __exit__(self, exc_type, exc_val, traceback):
         os.kill(self.pid, SIGKILL)
@@ -58,54 +42,17 @@
         else:
             self.send_error(httplib.BAD_REQUEST)
 
-    def do_GET(self):
-        scheme, netloc, path, params, query_string, fragments = (
-            urlparse(self.path))
-        archive = os.path.split(path)[1]
-        query = parse_qs(query_string)
-        parameters = simplejson.loads(query['parameters'][0])
-        self.send_response(httplib.OK)
-        self.end_headers()
-        messages = [m for m in self.server.messages[archive] if 'message_ids'
-                    not in parameters or m['message-id'] in
-                    parameters['message_ids']]
-        self.wfile.write(simplejson.dumps(messages))
-
-
-def fake_grackle_service(client, messages=None):
-    if messages is None:
-        messages = {}
-    return ForkedFake(client.port, messages)
+
+def run_service():
+    service = HTTPServer(('', 8435), FakeGrackleRequestHandler)
+    service.serve_forever()
+
 
 
 class TestPutMessage(TestCase):
 
     def test_put_message(self):
-        client = GrackleClient('localhost', 8436)
-        with fake_grackle_service(client):
-            client.put_message('arch1', 'asdf', StringIO('This is a message'))
+        with Forked(run_service):
+            client.put_message('arch1', StringIO('This is a message'))
             with ExpectedException(Exception, 'wtf'):
-                client.put_message('arch1', 'asdf',
-                    StringIO('This is not a message'))
-
-
-class TestGetMessages(TestCase):
-
-    def test_get_messages(self):
-        client = GrackleClient('localhost', 8435)
-        with fake_grackle_service(client,
-            {'baz':
-            [{'message-id': 'foo'},
-             {'message-id': 'bar'}]}):
-            response = client.get_messages('baz')
-        self.assertEqual(['bar', 'foo'], sorted(m['message-id'] for m in
-            response))
-
-    def test_get_messages_by_id(self):
-        client = GrackleClient('localhost', 8437)
-        with fake_grackle_service(client,
-            {'baz':
-            [{'message-id': 'foo'},
-             {'message-id': 'bar'}]}):
-            message, = client.get_messages('baz', message_ids=['foo'])
-        self.assertEqual('foo', message['message-id'])
+                client.put_message('arch1', StringIO('This is not a message'))

